@@ -18,6 +18,7 @@ class TaskPublisher(Publisher):
     exchange = conf.AMQP_EXCHANGE
     exchange_type = conf.AMQP_EXCHANGE_TYPE
     routing_key = conf.AMQP_PUBLISHER_ROUTING_KEY
+    serializer = "pickle"
     encoder = pickle.dumps
 
     def delay_task(self, task_name, task_args, task_kwargs, **kwargs):
@@ -25,8 +26,8 @@ class TaskPublisher(Publisher):
         return self._delay_task(task_name=task_name, task_args=task_args,
                                 task_kwargs=task_kwargs, **kwargs)
 
-    def delay_task_in_set(self, task_name, taskset_id, task_args,
-            task_kwargs, **kwargs):
+    def delay_task_in_set(self, taskset_id, task_name, task_args, task_kwargs,
+            **kwargs):
         """Delay a task which part of a task set."""
         return self._delay_task(task_name=task_name, part_of_set=taskset_id,
                                 task_args=task_args, task_kwargs=task_kwargs,
@@ -42,6 +43,7 @@ class TaskPublisher(Publisher):
     def _delay_task(self, task_name, task_id=None, part_of_set=None,
             task_args=None, task_kwargs=None, **kwargs):
         """INTERNAL"""
+        eta = kwargs.get("eta")
         priority = kwargs.get("priority")
         immediate = kwargs.get("immediate")
         mandatory = kwargs.get("mandatory")
@@ -55,6 +57,7 @@ class TaskPublisher(Publisher):
             "task": task_name,
             "args": task_args,
             "kwargs": task_kwargs,
+            "eta": eta,
         }
         if part_of_set:
             message_data["taskset"] = part_of_set
@@ -73,3 +76,21 @@ class TaskConsumer(Consumer):
     decoder = pickle.loads
     auto_ack = False
     no_ack = False
+
+
+class StatsPublisher(Publisher):
+    exchange = "celerygraph"
+    routing_key = "stats"
+    encoder = pickle.dumps
+
+
+class StatsConsumer(Consumer):
+    queue = "celerygraph"
+    exchange = "celerygraph"
+    routing_key = "stats"
+    exchange_type = "direct"
+    decoder = pickle.loads
+    no_ack=True
+
+    def receive(self, message_data, message):
+        pass
