@@ -5,8 +5,12 @@ Asynchronous result types.
 """
 from celery.backends import default_backend
 from celery.datastructures import PositionQueue
-from celery.timer import TimeoutTimer
 from itertools import imap
+import threading
+
+
+class TimeoutError(Exception):
+    """The operation timed out."""
 
 
 class BaseAsyncResult(object):
@@ -28,6 +32,8 @@ class BaseAsyncResult(object):
 
     """
 
+    TimeoutError = TimeoutError
+
     def __init__(self, task_id, backend):
         self.task_id = task_id
         self.backend = backend
@@ -50,7 +56,7 @@ class BaseAsyncResult(object):
         :keyword timeout: How long to wait in seconds, before the
             operation times out.
 
-        :raises celery.timer.TimeoutError: if ``timeout`` is not ``None`` and
+        :raises TimeoutError: if ``timeout`` is not ``None`` and
             the result does not arrive within ``timeout`` seconds.
 
         If the remote call raised an exception then that
@@ -144,31 +150,27 @@ class AsyncResult(BaseAsyncResult):
 class TaskSetResult(object):
     """Working with :class:`celery.task.TaskSet` results.
 
-    An instance of this class is returned by :meth:`celery.task.TaskSet.run()`.
-    It lets you inspect the status and return values of a taskset as a
-    single entity.
+    An instance of this class is returned by
+    :meth:`celery.task.TaskSet.run()`. It lets you inspect the status and
+    return values of the taskset as a single entity.
 
     :option taskset_id: see :attr:`taskset_id`.
-    :option subtask_ids: see :attr:`subtask_ids`.
+    :option subtasks: see :attr:`subtasks`.
+
 
     .. attribute:: taskset_id
 
         The UUID of the taskset itself.
 
-    .. attribute:: subtask_ids
-
-        The list of task UUID's for all of the subtasks.
-
     .. attribute:: subtasks
 
-        A list of :class:`AsyncResult`` instances for all of the subtasks.
+        A list of :class:`AsyncResult` instances for all of the subtasks.
 
     """
 
-    def __init__(self, taskset_id, subtask_ids):
+    def __init__(self, taskset_id, subtasks):
         self.taskset_id = taskset_id
-        self.subtask_ids = subtask_ids
-        self.subtasks = map(AsyncResult, self.subtask_ids)
+        self.subtasks = subtasks
 
     def itersubtasks(self):
         """Taskset subtask iterator.
@@ -239,8 +241,8 @@ class TaskSetResult(object):
         :raises: The exception if any of the tasks raised an exception.
 
         """
-        results = dict([(task_id, AsyncResult(task_id))
-                            for task_id in self.subtask_ids])
+        results = dict([(subtask.task_id, AsyncResult(subtask.task_id))
+                            for subtask in self.subtasks])
         while results:
             for task_id, pending_result in results.items():
                 if pending_result.status == "DONE":
@@ -257,7 +259,7 @@ class TaskSetResult(object):
         :keyword timeout: The time in seconds, how long
             it will wait for results, before the operation times out.
 
-        :raises celery.timer.TimeoutError: if ``timeout`` is not ``None``
+        :raises TimeoutError: if ``timeout`` is not ``None``
             and the operation takes longer than ``timeout`` seconds.
 
         If any of the tasks raises an exception, the exception
@@ -266,22 +268,27 @@ class TaskSetResult(object):
         :returns: list of return values for all tasks in the taskset.
 
         """
-        timeout_timer = TimeoutTimer(timeout)
+
+        def on_timeout():
+            raise TimeoutError("The operation timed out.")
+
+        timeout_timer = threading.Timer(timeout, on_timeout)
         results = PositionQueue(length=self.total)
 
-        while True:
-            for position, pending_result in enumerate(self.subtasks):
-                if pending_result.status == "DONE":
-                    results[position] = pending_result.result
-                elif pending_result.status == "FAILURE":
-                    raise pending_result.result
-            if results.full():
-                # Make list copy, so the returned type is not a position
-                # queue.
-                return list(results)
-
-            # This raises TimeoutError when timed out.
-            timeout_timer.tick()
+        timeout_timer.start()
+        try:
+            while True:
+                for position, pending_result in enumerate(self.subtasks):
+                    if pending_result.status == "DONE":
+                        results[position] = pending_result.result
+                    elif pending_result.status == "FAILURE":
+                        raise pending_result.result
+                if results.full():
+                    # Make list copy, so the returned type is not a position
+                    # queue.
+                    return list(results)
+        finally:
+            timeout_timer.cancel()
 
     @property
     def total(self):
