@@ -8,7 +8,7 @@ from celery.registry import tasks
 
 class TestPeriodicTask(PeriodicTask):
     name = "celery.unittest.test_models.test_periodic_task"
-    run_every = timedelta(days=1)
+    run_every = timedelta(minutes=30)
 
 
 class TestModels(unittest.TestCase):
@@ -58,7 +58,8 @@ class TestModels(unittest.TestCase):
         self.assertFalse(p in PeriodicTaskMeta.objects.get_waiting_tasks())
         # Have to avoid save() because it applies the auto_now=True.
         PeriodicTaskMeta.objects.filter(name=p.name).update(
-                last_run_at=datetime.now() - TestPeriodicTask.run_every)
+                last_run_at=datetime.now() - (TestPeriodicTask.run_every +
+                timedelta(seconds=10)))
         self.assertTrue(p in PeriodicTaskMeta.objects.get_waiting_tasks())
         self.assertTrue(isinstance(p.task, TestPeriodicTask))
 
